/*Shader

Author: Leonard Frankel

See Shader.NSF for comments*/

sampler2D BaseSampler : register(s0);
sampler2D PeakSampler : register(s1);

float4x4 WorldViewProj;
float4x4 WorldTransform;

//directional light:
float4 dirDirection;
float4 dirAmbient;
float4 dirDiffuse;

//point light 0:
float4 point0Position;
float4 point0Ambient;
float4 point0Diffuse;
float3 point0Attenuation;

//point light 1:
float4 point1Position;
float4 point1Ambient;
float4 point1Diffuse;
float3 point1Attenuation;

struct VS_OUTPUT
{
	float4 pos		: POSITION;
	float2 texCoord	: TEXCOORD;
	float4 ambient	: COLOR0; /*multiply r b g by texture colour to light it. alpha contains 
								height for height blending. only have two colour channels to
								work with!*/				
	float4 diffuse	: COLOR1; //multiple this by texture colour to light it
};

VS_OUTPUT VS(float4 inPos: POSITION, float2 inTex0: TEXCOORD0, float3 normal: NORMAL)
{
	VS_OUTPUT result;
	
	//transforming model to world coordinates:
	result.pos = mul(inPos, WorldViewProj);
	float4 vertexPos = mul(inPos, WorldTransform);
	float4 transformedNormal = mul(normal, WorldTransform);

	result.texCoord = inTex0;

	//ambient is as is
	result.ambient = dirAmbient + point0Ambient + point1Ambient;
	result.ambient.a = clamp(inPos.z, 0, 1);

	//diffuse is saturate( dot( reversedNormalisedLightDir, normal ) ) * diffuse
	float3 reversedDir = {dirDirection.x * -1,
						  dirDirection.y * -1,
						  dirDirection.z * -1};

	float4 dirDiffuseResult = dirDiffuse * saturate(dot( normalize(reversedDir), transformedNormal ));

	//point light 0:
	reversedDir = point0Position - vertexPos;
		/*by the way, the result of getting this reversed is VERY interesting - a light that
		  emits darkness!*/
	float reversedDirLength = length(reversedDir);
	float4 point0DiffuseResult = point0Diffuse * saturate(dot( normalize(reversedDir), transformedNormal ));

	//applying attentuation:

	/*attenuated light is:
	  original light / (constant attenuation
	                    + linear attenuation / distance
						+ quadratic attenuation / distance^2)*/
	point0DiffuseResult = point0DiffuseResult / (point0Attenuation[0]
	   											+ point0Attenuation[1] * reversedDirLength
											   	+ point0Attenuation[2] * (reversedDirLength * reversedDirLength) );



	//point light 1:
	reversedDir = point1Position - vertexPos;
	reversedDirLength = length(reversedDir);

	float4 point1DiffuseResult = point1Diffuse * saturate(dot( normalize(reversedDir), normal ));

	//applying attentuation:
	point1DiffuseResult = point1DiffuseResult / (point1Attenuation[0]
	   											+ point1Attenuation[1] * reversedDirLength
											   	+ point1Attenuation[2] * (reversedDirLength * reversedDirLength));

	result.diffuse = dirDiffuseResult + point0DiffuseResult + point1DiffuseResult;

	return result;
}

float4 PS(VS_OUTPUT In) : COLOR
{
	//blending base texture and dark texture using z coordinate:
	float4 baseColour = tex2D(BaseSampler, In.texCoord).rgba;
	float4 darkColour = tex2D(PeakSampler, In.texCoord).rgba;
	float4 colour = lerp(baseColour, darkColour, In.ambient.a);

	//applying lighting:
	float4 result = colour * (In.ambient + In.diffuse);
	return result;
}	
